@@ -25,13 +25,14 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
 #include "gdkpixmap.h"
 #include "gdkdisplay.h"
+#include "gdkscreen.h"
 
 #include "gdkprivate-win32.h"
 
@@ -129,7 +130,7 @@ gdk_pixmap_impl_win32_get_size (GdkDrawable *drawable,
 }
 
 GdkPixmap*
-gdk_pixmap_new (GdkDrawable *drawable,
+_gdk_pixmap_new (GdkDrawable *drawable,
 		gint         width,
 		gint         height,
 		gint         depth)
@@ -347,7 +348,7 @@ static const unsigned char mirror[256] = {
 };
 
 GdkPixmap *
-gdk_bitmap_create_from_data (GdkDrawable *drawable,
+_gdk_bitmap_create_from_data (GdkDrawable *drawable,
 			     const gchar *data,
 			     gint         width,
 			     gint         height)
@@ -387,7 +388,7 @@ gdk_bitmap_create_from_data (GdkDrawable *drawable,
 }
 
 GdkPixmap*
-gdk_pixmap_create_from_data (GdkDrawable    *drawable,
+_gdk_pixmap_create_from_data (GdkDrawable    *drawable,
 			     const gchar    *data,
 			     gint            width,
 			     gint            height,
@@ -446,6 +447,18 @@ gdk_pixmap_foreign_new_for_display (GdkDisplay      *display,
   return gdk_pixmap_foreign_new (anid);
 }
 
+GdkPixmap *
+gdk_pixmap_foreign_new_for_screen (GdkScreen       *screen,
+				   GdkNativeWindow  anid,
+				   gint             width,
+				   gint             height,
+				   gint             depth)
+{
+  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
+
+  return gdk_pixmap_foreign_new (anid);
+}
+
 GdkPixmap*
 gdk_pixmap_foreign_new (GdkNativeWindow anid)
 {
